unit UI_Rect;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Windows, Variants, Forms, Controls, Graphics,
  Dialogs, ExtCtrls, StdCtrls, Buttons;

type

  { TForm1 }

  TForm1 = class(TForm)
    ellipseBtn: TBitBtn;
    Image1: TImage;
    rectBtn: TBitBtn;
    clearBtn: TBitBtn;
    PaintBox1: TPaintBox;
    Timer1: TTimer;
    procedure FormPaint(Sender: TObject);
    procedure PaintBox1Paint(Sender: TObject);
    procedure clearBtnClick(Sender: TObject);
    procedure rectBtnClick(Sender: TObject);
    procedure ellipseBtnClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);

  private
    FSegments : Integer;
  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

type Ttask = (tkRectangle,tkEllipse);

const bgcolor = cl3DFace;
      timerInterval = 20;
      pixelsperTick = 10;

var penwidth : byte;
    pencolor : dword;
    drawingBusy : boolean;  //true while painting
    task : TTask;           //instruct timer
//
//-- variabes for timer driven paints
//
    path : byte;
    rx1,ry1,rx2,ry2 : smallInt;//rectangle left top, right bottom
    sx,sy : smallInt;          //(x,y) of path
    px,py : smallInt;          //calculated for ellipse
    a,b,cx,cy : single;        //ellipse size, position

//--- low level helpers

procedure clearpaintbox;
begin
  with form1.PaintBox1.canvas do
  begin
   brush.Style := bsSolid;
   brush.Color := bgColor;
   form1.PaintBox1.canvas.FillRect(Height,Width,0,0);
  end;

end;

procedure Dot(x,y : smallInt);
//paint dot at (x,y) of paintbox1 using penwidth,pencolor
var x1,y1,x2,y2,i,j : smallInt;
begin
 x1 := x - ((penwidth-1) shr 1);
 x2 := x + (penwidth shr 1);
 y1 := y - ((penwidth-1) shr 1);
 y2 := y + (penwidth shr 1);
 for i := x1 to x2 do
  for j := y1 to y2 do
   form1.PaintBox1.Canvas.pixels[i,j] := pencolor;
end;

//-- painting step by step called by time outs

function SPrectangleStep : boolean;
//called by timeout
//paint pixelsperTick number of dots
//exit true if finished
var OK : boolean;
    n : word;
begin
 n := pixelsPerTick;
 result := false;
 repeat
   dot(sx,sy);
   repeat
    OK := true;                       //OK: no path change
    case path of
     1 : if sx < rx2 then inc(sx) else begin
                                        path := 2;
                                        OK := false;
                                       end;
     2 : if sy < ry2 then inc(sy) else begin
                                        path := 3;
                                        OK := false;
                                       end;
     3 : if sx > rx1 then dec(sx) else begin
                                        path := 4;
                                        OK := false;
                                       end;
     4 : if sy > ry1 then dec(sy) else result := true;
    end;//case
   until OK;
   dec(n);
  until (n=0) or result;
end;

function SPellipseStep : boolean;
//called by timeout
//paint pixelsperTick number of dots
//exit true if finished
var n : word;
    OK : boolean;
begin
 n := pixelsPerTick;
 result := false;
 repeat
  dot(px,py);
  repeat
   OK := true;           //false if path change
   case path of
     1 : if sx < rx2 then
          begin
           inc(sx);
           px := sx;
           py := round(cy-b/a*sqrt(sqr(a)-sqr(cx-sx)));
          end else begin
                    OK := false;
                    path := 2;
                   end;
    2 : if sy < ry2 then
         begin
          inc(sy);
          py := sy;
          px := round(cx+a/b*sqrt(sqr(b)-sqr(cy-sy)));
         end else begin
                   OK := false;
                   path := 3;
                   inc(sx);//prevent hole
                  end;
    3 : if sx > rx1 then
         begin
          dec(sx);
          px := sx;
          py := round(cy+b/a*sqrt(sqr(a)-sqr(cx-sx)));
         end else begin
                   OK := false;
                   path := 4;
                   inc(sy);//prevent hole
                  end;
    4 : if sy > ry1 then
         begin
          dec(sy);
          py := sy;
          px := round(cx-a/b*sqrt(sqr(b)-sqr(cy-sy)));
         end else result := true;
    end;//case
   until OK;
   dec(n);
  until (n=0) or result;
end;

//--main calls:  slow paint procedures

procedure slowpaintRectangle(x1,y1,x2,y2 : smallInt);
//penwidth, pencolor set before
begin
 task := tkRectangle;
 path := 1;
 drawingbusy := true;
 rx1 := x1;
 ry1 := y1;
 rx2 := x2;
 ry2 := y2;
 sx := x1;
 sy := y1;
 Form1.fsegments:=0;
 with form1.Timer1 do
  begin
   interval :=  timerinterval;
   enabled := true;
  end;
end;

procedure slowPaintEllipse(x1,y1,x2,y2 : smallInt);
//penwidth, pencolor set before
var sab : single;
begin
 task := tkEllipse;
 drawingbusy := true;
 path := 1;
 cx := (x1+x2)/2;
 cy := (y1+y2)/2;
 a := (x2-x1)/2;
 b := (y2-y1)/2;
 sab := sqrt(sqr(a)+sqr(b));
 rx1 := round(cx-sqr(a)/sab);
 ry1 := round(cy-sqr(b)/sab);
 rx2 := round(2*cx - rx1);
 ry2 := round(2*cy - ry1);
 sx := rx1;
 sy := ry1;
 px := sx;
 py := round(cy-b/a*sqrt(sqr(a)-sqr(cx-sx)));
 Form1.fsegments:=0;
 with form1.Timer1 do
  begin
   interval :=  timerinterval;
   enabled := true;
  end;
end;

//-- buttons



procedure TForm1.clearBtnClick(Sender: TObject);
//clear paintbox
begin
// path := 1;
 if drawingbusy then exit;

//
 clearpaintbox;

end;

procedure TForm1.rectBtnClick(Sender: TObject);
//slow draw fixed rectangle
begin
 if drawingBusy then exit;
//
 penwidth := 3;
 pencolor := $000000;
 slowpaintRectangle(60,60,640,640);
end;

procedure TForm1.ellipseBtnClick(Sender: TObject);
//slow draw fixed ellipse
begin
 if drawingBusy then exit;
//
 penwidth := 3;
 pencolor := $0000ff;
 slowpaintEllipse(10,10,590,590);
end;

//-- paint events

procedure TForm1.FormPaint(Sender: TObject);
// paint edge around paintbox on form canvas
var x1, y1, x2, y2 : word;
    i : byte;
begin

 with paintbox1 do
  begin
   y1 := Top-2;
   x2 := Left+Width+1;
   y2 := Top+Height+1;
  end;

end;


procedure TForm1.PaintBox1Paint(Sender: TObject);

var i,elips : Integer;
begin
 clearpaintbox;
 sx:=rx1;
 sy:=ry1;
 Path:=1;

 for I     := 0 to FSegments do SPrectangleStep;
 for elips := 0 to FSegments do SPellipseStep ;

end;

procedure TForm1.Timer1Timer(Sender: TObject);
var fin : boolean;
begin
 inc(FSegments);
 fin := false;
 case task of
  tkRectangle : fin := SPrectangleStep;
  tkEllipse   : fin := SPellipseStep;
 end;
 if fin then begin
              timer1.Enabled := false;
              drawingbusy := false;
             end;

end;



end.

